(**********************************************************************************
 * The PEDANTIC (Proof Engine for Deductive Automation using Non-deterministic
 * Traversal of Instruction Code) verification framework
 *
 * Developed by Kenneth Roe
 * For more information, check out www.cs.jhu.edu/~roe
 *
 * ProgramTactics.v
 * This file contains 'pcrunch'.  This tactic is an expansion of crunch that
 * handles a few definitions not present in SfLib and SfLibExtras.
 *
 **********************************************************************************)

Require Export SfLib.
Require Export SfLibExtras.
Require Export Coq.Logic.FunctionalExtensionality.
Require Export AbsState.
Require Export AbsStateInstance.
Require Export AbsExecute.
Require Export Simplify.
Require Export Eqdep.
Require Export StateImplication.
Require Export Classical.
Require Export Unfold.
Require Export merge.

Ltac pcrunchStep := match goal with
                   | [ |- _ ] => progress crunchStep
                   | [ |- _ ] => progress auto

                   | [ H: NatValue _ = NoValue |- _ ] => inversion H
                   | [ H: ListValue _ = NoValue |- _ ] => inversion H
                   | [ H: OtherValue _ = NoValue |- _ ] => inversion H
                   | [ H: NoValue = NatValue _ |- _ ] => inversion H
                   | [ H: ListValue = NatValue _ |- _ ] => inversion H
                   | [ H: OtherValue = NatValue _ |- _ ] => inversion H
                   | [ H: NoValue = ListValue _ |- _ ] => inversion H
                   | [ H: NatValue = ListValue _ |- _ ] => inversion H
                   | [ H: OtherValue = ListValue _ |- _ ] => inversion H
                   | [ H: NoValue = OtherValue _ |- _ ] => inversion H
                   | [ H: NatValue = OtherValue _ |- _ ] => inversion H
                   | [ H: ListValue = OtherValue _ |- _ ] => inversion H

                   | [ |- NatValue _ <> NoValue ] => let x :=fresh in (intro x;inversion x)
                   | [ |- ListValue _ <> NoValue ] => let x :=fresh in (intro x;inversion x)
                   | [ |- OtherValue _ <> NoValue ] => let x :=fresh in (intro x;inversion x)
                   | [ |- NoValue <> NatValue _ ] => let x :=fresh in (intro x;inversion x)
                   | [ |- ListValue <> NatValue _ ] => let x :=fresh in (intro x;inversion x)
                   | [ |- OtherValue _ <> NatValue ] => let x :=fresh in (intro x;inversion x)
                   | [ |- NoValue <> ListValue _ ] => let x :=fresh in (intro x;inversion x)
                   | [ |- NatValue <> ListValue _ ] => let x :=fresh in (intro x;inversion x)
                   | [ |- OtherValue _ <> ListValue ] => let x :=fresh in (intro x;inversion x)
                   | [ |- NoValue <> OtherValue _ ] => let x :=fresh in (intro x;inversion x)
                   | [ |- NatValue <> OtherValue _ ] => let x :=fresh in (intro x;inversion x)
                   | [ |- ListValue _ <> OtherValue ] => let x :=fresh in (intro x;inversion x)

                   | [ H: NatValue _ = NatValue _ |- _ ] => (inversion H;subst;clear H)
                   | [ H: ListValue _ = ListValue _ |- _ ] => (inversion H;subst;clear H)
                   | [ H: OtherValue _ = OtherValue _ |- _ ] => (inversion H;subst;clear H)

                   | [ |- validExpression _ _ ] => unfold validExpression
                   end.

Ltac pcrunch := repeat pcrunchStep.
